@@ -1,9 +1,10 @@
+# noqa: N999
 """
 HinetPy
 =======
 
-HinetPy is a Python package to request and process seismic waveform data
-from the NIED Hi-net website.
+HinetPy is a Python package to request and process seismic waveform data from the NIED
+Hi-net website.
 
 Basis usage:
 
@@ -16,13 +17,12 @@ Basis usage:
 >>> win32.extract_sac(data, ctable)
 >>> win32.extract_sacpz(ctable)
 """
-# pylint: disable=invalid-name
 
-from pkg_resources import get_distribution
+from importlib.metadata import version
 
 from .client import Client
 from .header import NETWORK
 
 __all__ = ["Client", "NETWORK", "win32"]
 # Get semantic version through setuptools-scm
-__version__ = f'v{get_distribution("HinetPy").version}'  # e.g. v0.1.2.dev3+g0ab3cd78
+__version__ = f'v{version("HinetPy")}'  # e.g. v0.1.2.dev3+g0ab3cd78
